# SignedDistanceField

[![Dev](https://img.shields.io/badge/docs-dev-blue.svg)](https://pierrat.pages.emse.fr/SignedDistanceField.jl/dev)
[![Build Status](https://gitlab.emse.fr/pierrat/SignedDistanceField.jl/badges/master/pipeline.svg)](https://gitlab.emse.fr/pierrat/SignedDistanceField.jl/pipelines)
[![Coverage](https://gitlab.emse.fr/pierrat/SignedDistanceField.jl/badges/master/coverage.svg)](https://gitlab.emse.fr/pierrat/SignedDistanceField.jl/commits/master)


 `SignedDistanceField.jl` is a package written in Julia to compute the signed distance field of a triangular mesh over a regular grid.

-----------------------------

## Basic usage

```
s, x, y, z = sdfgen("test/cube.obj", 0.1)
[ Info: The bounding box of the mesh is: -1.1:1.1, -1.1:1.1, -1.1:1.1
[ Info: Signed distance will be computed on a 23 x 23 x 23 grid

display_sdf(s, x, y, z)
```

![Cube](docs/cube.png)

```
s, x, y, z = sdfgen("test/bunny.obj", 0.001)
[ Info: The bounding box of the mesh is: -0.095380425:0.061619584, 0.0323099:0.1873099, -0.06267917:0.059320834
[ Info: Signed distance will be computed on a 158 x 156 x 123 grid
Computing signed distance field... 100%|██████████████████████████| Time: 0:02:20

display_sdf(s, x, y, z)
```

![Bunny](docs/bunny.png)

## KDTree acceleration

When the number of triangles in the mesh is large, the search for the closest triangle can be accelerated using a KDTree:
```
@time sdfgen("test/bunny.obj", 0.001; acceleration=nothing);
135.721059 seconds

@time sdfgen("test/bunny.obj", 0.001; acceleration=:KDTree);
1.931617 seconds
```

## Notes:
- The underlying algorithm (signed distance computation using pseudonormals) assumes that the mesh is watertight (no holes).
- KDTree acceleration assumes that the mesh size is rather homogeneous. Indeed, it looks for 10 candidates for the closest triangle in the KDTree based on the barycenter of each triangle.

## References
J. Andreas Baerentzen and Henrik Aanaes: Generating Signed Distance Fields From Triangle Meshes, Technical Report, IMM. 2002.

Christer Ericson: Real-Time Collision Detection. 2005
