struct StructArrayView{T,Ti}
    structarray::T
    idxs::Ti
end

import Base.getproperty, Base.setproperty!, Base.show

@inline Base.@propagate_inbounds function Base.getproperty(c::StructArrayView, s::Symbol)
    return getproperty(getfield(c, 1), s)[getfield(c, 2)]
end

@inline Base.@propagate_inbounds function Base.setproperty!(c::StructArrayView, s::Symbol, val)
    getproperty(getfield(c, 1), s)[getfield(c, 2)] = val
end

function Base.show(io::IO, c::StructArrayView)
    print(io, "StructArrayView: ")
    structarray, idxs = getfield(c, 1), getfield(c, 2)
    map(i -> show(io, StructArray(fieldarrays(structarray))[i]), idxs)
end

struct Vertex
    idx::Int
    position::Vec3f
    normal::Vec3f
end

create_single_vertex(idx, position::AbstractVector) = Vertex(idx, position, Vec3f(0))

function create_single_edge(idx, points::Tpoints, pointsidx) where {Tpoints}
    ptsv = StructArrayView{Tpoints,Vec2{Int}}(points, pointsidx)
    return Edge{Tpoints}(idx, ptsv, Vec3f(0))
end

struct Edge{Tpoint}
    idx::Int
    points::StructArrayView{Tpoint,Vec2{Int}}
    normal::Vec3f
end

struct Triangle{Tpoint,Tedge}
    idx::Int
    points::StructArrayView{Tpoint,Vec3{Int}}
    edges::StructArrayView{Tedge,Vec3{Int}}
    normal::Vec3f
end

function create_single_triangle(idx, points::Tpoints, pointsidx, edges::Tedges, edgesidx) where {Tpoints,Tedges}

    pos = points.position[pointsidx]

    a = pos[1]
    b = pos[2]
    c = pos[3]
    n = cross(b - a, c - a)
    n = normalize(n)

    ptsv = StructArrayView{Tpoints,Vec3{Int}}(points, pointsidx)
    edgv = StructArrayView{Tedges,Vec3{Int}}(edges, edgesidx)

    return Triangle{Tpoints,Tedges}(idx, ptsv, edgv, n)
end



# Generates edge connectivity from triangle connectivity
function populate_edges(trisconn)

    edges = Vector{Tuple{Int,Int}}()
    edge_idx = 0
    edges_in_tri = [[0, 0, 0] for _ = 1:length(trisconn)]

    for (idx, t) in enumerate(trisconn)
        for i = 1:3

            pt1 = t[i]
            pt2 = t[mod1(i + 1, 3)]

            edge = (min(pt1, pt2), max(pt1, pt2))

            edge_idx_a = findfirst(isequal(edge), edges)
            if isnothing(edge_idx_a)
                push!(edges, edge)
                edge_idx = length(edges)
            else
                edge_idx = edge_idx_a
            end

            edges_in_tri[idx][i] = edge_idx

        end
    end

    return edges, edges_in_tri

end
